@@ -14,15 +14,14 @@ import mimetypes
 import time
 import cgi
 import os
-import webbrowser
-from Cookie import BaseCookie
+from Cookie import BaseCookie, CookieError
 try:
     from cStringIO import StringIO
 except ImportError:
     from StringIO import StringIO
 import re
 from webob import Response, Request
-from wsgiref.validate import validator
+from webtest import lint
 
 __all__ = ['TestApp']
 
@@ -291,7 +290,7 @@ class TestApp(object):
             req.environ['HTTP_COOKIE'] = str(c).split(': ', 1)[1]
         req.environ['paste.testing'] = True
         req.environ['paste.testing_variables'] = {}
-        app = validator(self.app)
+        app = lint.middleware(self.app)
         old_stdout = sys.stdout
         out = CaptureStdout(old_stdout)
         try:
@@ -321,7 +320,11 @@ class TestApp(object):
             self._check_errors(res)
         res.cookies_set = {}
         for header in res.headers.getall('set-cookie'):
-            c = BaseCookie(header)
+            try:
+                c = BaseCookie(header)
+            except CookieError, e:
+                raise CookieError(
+                    "Could not parse cookie header %r: %s" % (header, e))
             for key, morsel in c.items():
                 self.cookies[key] = morsel.value
                 res.cookies_set[key] = morsel.value
@@ -844,6 +847,7 @@ class TestResponse(Response):
         Show this response in a browser window (for debugging purposes,
         when it's hard to read the HTML).
         """
+        import webbrowser
         fn = tempnam_no_warning(None, 'webtest-page') + '.html'
         f = open(fn, 'wb')
         f.write(self.body)
@@ -942,7 +946,10 @@ class Form(object):
             tag_type = tag
             if tag == 'input':
                 tag_type = attrs.get('type', 'text').lower()
-            FieldClass = Field.classes.get(tag_type, Field)
+            if tag_type == "select" and attrs.get("multiple"):
+                FieldClass = Field.classes.get("multiple_select", Field)
+            else:
+                FieldClass = Field.classes.get(tag_type, Field)
             field = FieldClass(self, tag, name, match.start(), **attrs)
             if tag == 'textarea':
                 assert not in_textarea, (
@@ -1071,11 +1078,17 @@ class Form(object):
             field = self.get(name, index=index)
             submit.append((field.name, field.value_if_submitted()))
         for name, fields in self.fields.items():
+            if name is None:
+                continue
             for field in fields:
                 value = field.value
                 if value is None:
                     continue
-                submit.append((name, value))
+                if isinstance(value, list):
+                    for item in value:
+                        submit.append((name, item))
+                else:
+                    submit.append((name, value))
         return submit
 
 
@@ -1139,9 +1152,6 @@ class Select(Field):
     def __init__(self, *args, **attrs):
         super(Select, self).__init__(*args, **attrs)
         self.options = []
-        self.multiple = attrs.get('multiple')
-        assert not self.multiple, (
-            "<select multiple> not yet supported")
         # Undetermined yet:
         self.selectedIndex = None
 
@@ -1173,6 +1183,49 @@ class Select(Field):
 
 Field.classes['select'] = Select
 
+class MultipleSelect(Field):
+
+    """
+    Field representing ``<select multiple="multiple">``
+    """
+
+    def __init__(self, *args, **attrs):
+        super(MultipleSelect, self).__init__(*args, **attrs)
+        self.options = []
+        # Undetermined yet:
+        self.selectedIndices = []
+
+    def value__set(self, values):
+        str_values = [str(value) for value in values]
+        self.selectedIndicies = []
+        for i, (option, checked) in enumerate(self.options):
+            if option in str_values:
+                self.selectedIndices.append(i)
+                str_values.remove(option)
+        if str_values:
+            raise ValueError(
+                "Option(s) %r not found (from %s)"
+                % (', '.join(str_values),
+                   ', '.join(
+                        [repr(o) for o, c in self.options])))
+
+    def value__get(self):
+        if self.selectedIndices:
+            return [self.options[i][0] for i in self.selectedIndices]
+        else:
+            selected_values = []
+            for option, checked in self.options:
+                if checked:
+                    selected_values.append(option)
+                
+            if self.options and (not selected_values):
+                selected_values = None
+            return selected_values
+    
+    value = property(value__get, value__set)
+
+Field.classes['multiple_select'] = MultipleSelect
+
 class Radio(Select):
 
     """
@@ -1197,8 +1250,7 @@ class Checkbox(Field):
     def value__get(self):
         if self.checked:
             if self._value is None:
-                # @@: 'on'?
-                return 'checked'
+                return 'on'
             else:
                 return self._value
         else:
